package de.ovgu.dke.mocca.daemon;

import org.apache.commons.daemon.DaemonContext;
import org.apache.commons.daemon.DaemonController;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import de.ovgu.dke.mocca.api.MoccaException;

/**
 * The runtime shell for MOCCA daemons.
 * 
 * @author Stefan Haun (stefan.haun@ovgu.de)
 * 
 */
public class MoccaShell implements DaemonContext, DaemonController, Runnable {
	static Log logger = LogFactory.getLog(MoccaShell.class);

	final String[] args;
	final String agentClass;
	final MoccaDaemon daemon;

	public MoccaShell(final String[] args, String agentClass)
			throws MoccaException {
		this.args = args;
		this.agentClass = agentClass;
		this.daemon = new MoccaDaemon();

	}

	public void init() throws MoccaException {
		try {
			this.daemon.init(this);
		} catch (Exception e) {
			throw new MoccaException("Error during daemon initialization: "
					+ e.getMessage(), e);
		}

		try {
			daemon.loadAgent(agentClass);
		} catch (MoccaException e) {
			throw new MoccaException("Error loading agent class: "
					+ e.getMessage(), e);
		}
	}

	@Override
	public void run() {

		try {
			daemon.start();
		} catch (Exception e) {
			fail("Could not start Mocca Daemon: " + e.getMessage(), e);
		}
	}

	/**
	 * Init and start a Mocca Daemon running in this shell. The fully-qualified
	 * Daemon class path must be provided as the first and only parameter.
	 * 
	 * @param args
	 *            Command line args
	 */
	public static void main(String[] args) {
		if (args.length != 1) {
			System.out
					.println("Usage: MoccaShell <FQ ClassPath to Mocca Agent>");
			return;
		}

		try {
			final String path = args[0];
			final MoccaShell shell = new MoccaShell(args, path);

			shell.init();

			shell.run();

			// shutdown the daemon when finished.
			Runtime.getRuntime().addShutdownHook(new Thread(new Runnable() {

				@Override
				public void run() {
					shell.shutdown();
				}

			}));

			System.out.println("Press Ctrl-c to quit.");
			while (true) {
				try {
					Thread.sleep(1000);
				} catch (InterruptedException e) {
					break;
				}
			}

		} catch (MoccaException e) {
			logger.fatal("Fatal error running Mocca Shell: " + e.getMessage(),
					e);
		}
	}

	/**
	 * Run the shell with a specific agent class.
	 * 
	 * @param daemonClass
	 *            Fully-qualified class path.
	 */
	public static void start(final String daemonClass) {
		main(new String[] { daemonClass });
	}

	/**
	 * Run the shell with a specific agent class.
	 * 
	 * @param daemonClass
	 *            agent class object.
	 */
	public static void start(final Class<?> daemonClass) {
		main(new String[] { daemonClass.getName() });
	}

	@Override
	public String[] getArguments() {
		return args;
	}

	@Override
	public DaemonController getController() {
		return this;
	}

	@Override
	public void fail() throws IllegalStateException {
		logger.fatal("Mocca Daemon Shell failed (without message).");

	}

	@Override
	public void fail(String msg) throws IllegalStateException {
		logger.fatal("Mocca Daemon Shell failed: " + msg);

	}

	@Override
	public void fail(Exception e) throws IllegalStateException {
		logger.fatal("Mocca Daemon Shell failed: " + e.getMessage(), e);

	}

	@Override
	public void fail(String msg, Exception e) throws IllegalStateException {
		logger.fatal("Mocca Daemon Shell failed: " + msg, e);
	}

	@Override
	public void reload() throws IllegalStateException {
		try {
			daemon.stop();
			daemon.start();
		} catch (Exception e) {
			this.fail(e);
		}
	}

	@Override
	public void shutdown() throws IllegalStateException {
		if (daemon.getState().equals(MoccaDaemon.State.Running))
			try {
				daemon.stop();
			} catch (Exception e) {
				logger.warn("Error on stopping the daemon: " + e.getMessage(),
						e);
			}
		daemon.destroy();
	}

}
